@@ -1,6 +1,6 @@
 ;;; consult-imenu.el --- Consult commands for imenu -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021, 2022  Free Software Foundation, Inc.
+;; Copyright (C) 2021-2023 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -35,10 +35,11 @@
                              (?v "Variables" font-lock-variable-name-face))))
   "Imenu configuration, faces and narrowing keys used by `consult-imenu'.
 
-For each type a narrowing key and a name must be specified. The face is
-optional. The imenu representation provided by the backend usually puts
-functions directly at the toplevel. `consult-imenu' moves them instead under the
-type specified by :toplevel."
+For each type a narrowing key and a name must be specified.  The
+face is optional.  The imenu representation provided by the
+backend usually puts functions directly at the toplevel.
+`consult-imenu' moves them instead under the type specified by
+:toplevel."
   :type '(repeat (cons symbol plist))
   :group 'consult)
 
@@ -157,7 +158,17 @@ TYPES is the mode-specific types configuration."
 
 (defun consult-imenu--multi-items (buffers)
   "Return all imenu items from BUFFERS."
-  (apply #'append (consult--buffer-map buffers #'consult-imenu--items-safe)))
+  (consult--with-increased-gc
+   (let ((reporter (make-progress-reporter "Collecting" 0 (length buffers))))
+     (prog1
+         (apply #'append
+                (seq-map-indexed (lambda (buf idx)
+                                   (with-current-buffer buf
+                                     (prog1 (consult-imenu--items-safe)
+                                       (progress-reporter-update
+                                        reporter (1+ idx) (buffer-name)))))
+                                 buffers))
+       (progress-reporter-done reporter)))))
 
 (defun consult-imenu--jump (item)
   "Jump to imenu ITEM via `consult--jump'.
@@ -221,22 +232,25 @@ this function can jump across buffers."
 (defun consult-imenu ()
   "Select item from flattened `imenu' using `completing-read' with preview.
 
-The command supports preview and narrowing. See the variable
+The command supports preview and narrowing.  See the variable
 `consult-imenu-config', which configures the narrowing.
 The symbol at point is added to the future history.
 
 See also `consult-imenu-multi'."
   (interactive)
-  (consult-imenu--select "Go to item: " (consult-imenu--items)))
+  (consult-imenu--select
+   "Go to item: "
+   (consult--slow-operation "Building Imenu..."
+     (consult-imenu--items))))
 
 ;;;###autoload
 (defun consult-imenu-multi (&optional query)
   "Select item from the imenus of all buffers from the same project.
 
 In order to determine the buffers belonging to the same project, the
-`consult-project-function' is used. Only the buffers with the
-same major mode as the current buffer are used. See also
-`consult-imenu' for more details. In order to search a subset of buffers,
+`consult-project-function' is used.  Only the buffers with the
+same major mode as the current buffer are used.  See also
+`consult-imenu' for more details.  In order to search a subset of buffers,
 QUERY can be set to a plist according to `consult--buffer-query'."
   (interactive "P")
   (unless (keywordp (car-safe query))
