@@ -1,6 +1,6 @@
 ;;; consult-org.el --- Consult commands for org-mode -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021, 2022  Free Software Foundation, Inc.
+;; Copyright (C) 2021-2023 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -59,23 +59,30 @@
 
 If PREFIX is non-nil, prefix the candidates with the buffer name.
 MATCH, SCOPE and SKIP are as in `org-map-entries'."
-  (let (buffer)
+  (let (buffer (idx 0))
     (apply
      #'org-map-entries
      (lambda ()
-        ;; Reset the cache when the buffer changes, since `org-get-outline-path' uses the cache
+       ;; Reset the cache when the buffer changes, since `org-get-outline-path' uses the cache
        (unless (eq buffer (buffer-name))
          (setq buffer (buffer-name)
                org-outline-path-cache nil))
-       (pcase-let ((`(_ ,level ,todo ,prio ,_hl ,tags) (org-heading-components))
-                   (cand (org-format-outline-path
-                          (org-get-outline-path 'with-self 'use-cache)
-                          most-positive-fixnum)))
+       (pcase-let* ((`(_ ,level ,todo ,prio ,_hl ,tags) (org-heading-components))
+                    (tags (if org-use-tag-inheritance
+                              (when-let ((tags (org-get-tags)))
+                                (concat ":" (string-join tags ":") ":"))
+                            tags))
+                    (cand (org-format-outline-path
+                           (org-get-outline-path 'with-self 'use-cache)
+                           most-positive-fixnum)))
          (when tags
-           (setq tags (concat " " (propertize tags 'face 'org-tag))))
+           (put-text-property 0 (length tags) 'face 'org-tag tags))
          (setq cand (if prefix
-                        (concat buffer " " cand tags (consult--tofu-encode (point)))
-                      (concat cand tags (consult--tofu-encode (point)))))
+                        (concat buffer " " cand (and tags " ")
+                                tags (consult--tofu-encode idx))
+                      (concat cand (and tags " ")
+                              tags (consult--tofu-encode idx))))
+         (cl-incf idx)
          (add-text-properties 0 1
                               `(consult--candidate ,(point-marker)
                                 consult-org--heading (,level ,todo . ,prio))
@@ -94,7 +101,9 @@ buffer are offered."
                  (user-error "Must be called from an Org buffer")))
   (let ((prefix (not (memq scope '(nil tree region region-start-level file)))))
     (consult--read
-     (consult-org--headings prefix match scope)
+     (consult--slow-operation "Collecting headings..."
+       (or (consult-org--headings prefix match scope)
+           (user-error "No headings")))
      :prompt "Go to heading: "
      :category 'consult-org-heading
      :sort nil
@@ -115,7 +124,7 @@ buffer are offered."
 (defun consult-org-agenda (&optional match)
   "Jump to an Org agenda heading.
 
-By default, all agenda entries are offered. MATCH is as in
+By default, all agenda entries are offered.  MATCH is as in
 `org-map-entries' and can used to refine this."
   (interactive)
   (unless org-agenda-files
