@@ -1,6 +1,6 @@
 ;;; consult-register.el --- Consult commands for registers -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021, 2022  Free Software Foundation, Inc.
+;; Copyright (C) 2021-2023 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -24,8 +24,9 @@
 ;;; Code:
 
 (require 'consult)
+(require 'kmacro)
 
-(defcustom consult-register-prefix #("@" 0 1 (face consult-key))
+(defcustom consult-register-prefix #("#" 0 1 (face consult-key))
   "Prepend prefix in front of register keys during completion."
   :type '(choice (const nil) string)
   :group 'consult)
@@ -38,9 +39,10 @@
     (?t . "Frameset")
     (?k . "Kmacro")
     (?f . "File")
+    (?b . "Buffer")
     (?w . "Window"))
   "Register type names.
-Each element of the list must have the form \\='(char . name).")
+Each element of the list must have the form (char . name).")
 
 (cl-defun consult-register--format-value (val)
   "Format generic register VAL as string."
@@ -64,26 +66,33 @@ Each element of the list must have the form \\='(char . name).")
 (cl-defmethod consult-register--describe ((val marker))
   "Describe marker register VAL."
   (with-current-buffer (marker-buffer val)
-    (save-restriction
-      (save-excursion
+    (save-excursion
+      (save-restriction
         (widen)
         (goto-char val)
         (let* ((line (line-number-at-pos))
-               (str (propertize (consult--line-with-cursor val)
+               (str (propertize (consult--line-with-mark val)
                                 'consult-location (cons val line))))
-          (list (consult--format-location (buffer-name) line str)
+          (list (consult--format-file-line-match (buffer-name) line str)
                 'multi-category `(consult-location . ,str)
                 'consult--type ?p))))))
 
-(cl-defmethod consult-register--describe ((val kmacro-register))
-  "Describe kmacro register VAL."
-  (list (consult-register--format-value val) 'consult--type ?k))
+(defmacro consult-register--describe-kmacro ()
+  "Generate method which describes kmacro register."
+  `(cl-defmethod consult-register--describe ((val ,(if (< emacs-major-version 30) 'kmacro-register 'kmacro)))
+     (list (consult-register--format-value val) 'consult--type ?k)))
+(consult-register--describe-kmacro)
 
 (cl-defmethod consult-register--describe ((val (head file)))
   "Describe file register VAL."
   (list (propertize (abbreviate-file-name (cdr val)) 'face 'consult-file)
         'consult--type ?f 'multi-category `(file . ,(cdr val))))
 
+(cl-defmethod consult-register--describe ((val (head buffer)))
+  "Describe buffer register VAL."
+  (list (propertize (cdr val) 'face 'consult-buffer)
+        'consult--type ?f 'multi-category `(buffer . ,(cdr val))))
+
 (cl-defmethod consult-register--describe ((val (head file-query)))
   "Describe file-query register VAL."
   (list (format "%s at position %d"
@@ -160,30 +169,36 @@ If COMPLETION is non-nil format the register for completion."
        str))
     str))
 
-(defun consult-register--alist (&optional noerror)
-  "Return sorted register list.
+(defun consult-register--alist (&optional noerror filter)
+  "Return register list, sorted and filtered with FILTER.
 Raise an error if the list is empty and NOERROR is nil."
-  ;; Sometimes, registers are made without a `cdr'.
-  ;; Such registers don't do anything, and can be ignored.
-  (or (sort (seq-filter #'cdr register-alist) #'car-less-than-car)
+  (or (sort (seq-filter
+             ;; Sometimes, registers are made without a `cdr'.
+             ;; Such registers don't do anything, and can be ignored.
+             (lambda (x) (and (cdr x) (or (not filter) (funcall filter x))))
+             register-alist)
+            #'car-less-than-car)
       (and (not noerror) (user-error "All registers are empty"))))
 
+(defun consult-register--candidates (&optional filter)
+  "Return formatted completion candidates, filtered with FILTER."
+  (mapcar (lambda (reg) (consult-register-format reg 'completion))
+          (consult-register--alist nil filter)))
+
 ;;;###autoload
 (defun consult-register (&optional arg)
   "Load register and either jump to location or insert the stored text.
 
-This command is useful to search the register contents. For quick access
+This command is useful to search the register contents.  For quick access
 to registers it is still recommended to use the register functions
 `consult-register-load' and `consult-register-store' or the built-in
-built-in register access functions. The command supports narrowing, see
-`consult-register--narrow'. Marker positions are previewed. See
+built-in register access functions.  The command supports narrowing, see
+`consult-register--narrow'.  Marker positions are previewed.  See
 `jump-to-register' and `insert-register' for the meaning of prefix ARG."
   (interactive "P")
   (consult-register-load
    (consult--read
-    (mapcar (lambda (reg)
-              (consult-register-format reg 'completion))
-            (consult-register--alist))
+    (consult-register--candidates)
     :prompt "Register: "
     :category 'multi-category
     :state
@@ -205,8 +220,8 @@ built-in register access functions. The command supports narrowing, see
 (defun consult-register-load (reg &optional arg)
   "Do what I mean with a REG.
 
-For a window configuration, restore it. For a number or text, insert it.
-For a location, jump to it. See `jump-to-register' and `insert-register'
+For a window configuration, restore it.  For a number or text, insert it.
+For a location, jump to it.  See `jump-to-register' and `insert-register'
 for the meaning of prefix ARG."
   (interactive
    (list
@@ -284,8 +299,8 @@ This function is derived from `register-read-with-preview'."
   "Store register dependent on current context, showing an action menu.
 
 With an active region, store/append/prepend the contents, optionally
-deleting the region when a prefix ARG is given. With a numeric prefix
-ARG, store or add the number. Otherwise store point, frameset, window or
+deleting the region when a prefix ARG is given.  With a numeric prefix
+ARG, store or add the number.  Otherwise store point, frameset, window or
 kmacro."
   (interactive "P")
   (consult-register--action
